import 'package:agora_rtc/FUManager/FUConfig.dart';
import 'package:agora_rtc/FUManager/FUManager.dart';
// import 'package:common_utils/common_utils.dart';
// import 'package:common_utils/common_utils.dart';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

typedef AnimatComplete = void Function();

class FUManagerView {
  static showFaceUnityView(BuildContext context,
      { AnimatComplete? animatComplete}) async {
    await showModalBottomSheet(
        context: context,
        isScrollControlled: true,
        backgroundColor: Colors.transparent,
        barrierColor: Colors.transparent,
        builder: (context) {
          return FUManagerViewWidget();
        }).then((value) => {
          if (animatComplete != null) {animatComplete()}
        });
  }
}

class FUManagerViewWidget extends StatefulWidget {
  // final Color? tabsColor;
  // FUManagerViewWidget({this.tabsColor});
  @override
  FUManagerViewWidgetState createState() => FUManagerViewWidgetState();
}

class FUManagerViewWidgetState extends State<FUManagerViewWidget>
    with SingleTickerProviderStateMixin {
  TabController? _tabController;
  List tabs = [
    'Skin',
    'Facial',
    'Filter',
    'Sticker',
    'Makeup',
    'Beautify Body'
  ];
  // int _tabsInt = 0;
  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        Expanded(
            child: GestureDetector(
                onTap: () {
                  Navigator.of(context).pop();
                },
                child: Container(
                  color: Colors.transparent,
                ))),
        Container(
            decoration: BoxDecoration(
                boxShadow: [
                  BoxShadow(
                      color: Color(0xff160B20).withAlpha(155),
                      offset: Offset(0, 0),
                      blurRadius: 14,
                      spreadRadius: 1),
                ],
                borderRadius: BorderRadius.only(
                    topLeft: Radius.circular(14),
                    topRight: Radius.circular(14)),
                color: Colors.transparent),
            child: Container(
              // color: Colors.transparent,
              height: 230 + ScreenUtil().bottomBarHeight,
              child: buildView(),
            ))
      ],
    );
  }

  Widget buildView() {
    // return Container(
    //   height: 200,
    // );
    return Column(children: [
      Container(
          height: 55,
          decoration: BoxDecoration(
            borderRadius: BorderRadius.only(
                topLeft: Radius.circular(7.5), topRight: Radius.circular(7.5)),
            color: Color(0xFFFFFFFF),
          ),
          child: Column(
            children: [
              Container(
                margin: EdgeInsets.only(top: 15),
                height: 25,
                child: TabBar(
                  tabs: tabs.map((e) {
                    return Tab(
                      child: Align(
                        alignment: Alignment.center,
                        child: Text(e),
                      ),
                    );
                  }).toList(),
                  controller: _tabController,
                  unselectedLabelStyle:
                      TextStyle(fontSize: 14, color: Color(0xFF666666)),
                  labelStyle: TextStyle(
                    fontSize: 18,
                    color: Color(0xFFFB5C9A),
                  ),
                  indicatorSize: TabBarIndicatorSize.label,
                  isScrollable: true,
                  labelColor: Color(0xFFFB5C9A),
                  unselectedLabelColor: Color(0xFF666666),
                  indicatorWeight: 2,
                  indicatorColor: Color(0xFFFB5C9A),
                  // indicatorPadding: EdgeInsets.only(bottom: 15),
                  onTap: (index) {
                    // DLog(index);
                    // _tabsInt = index;
                  },
                ),
              ),
            ],
          )),
      Expanded(
          child: TabBarView(
        controller: _tabController,
        children: [
          FCSkinManagerView(
            FUConfig.currentSkinList,
            clickDefineBtn: () {
              if (mounted) {
                setState(() {});
              }
            },
          ),
          FCSkinManagerView(
            FUConfig.currentFacialList,
            clickDefineBtn: () {
              if (mounted) {
                setState(() {});
              }
            },
          ),
          FCFilterManagerView(FUConfig.currentFiltersList),
          FCFilterManagerView(FUConfig.currentStickerList),
          FCFilterManagerView(FUConfig.currentMakeupsList),
          FCSkinManagerView(
            FUConfig.currentBeautifyBodyList,
            clickDefineBtn: () {
              if (mounted) {
                setState(() {});
              }
            },
          ),
        ],
      ))
    ]);
  }

  @override
  void initState() {
    if (FUConfig.currentSkinList.isEmpty) {
      FUConfig.initialize();
    }
    super.initState();
    _tabController = TabController(vsync: this, length: 6);
  }

  @override
  void dispose() {
    // TODO: implement dispose
    _tabController?.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(FUManagerViewWidget oldWidget) {
    // TODO: implement didUpdateWidget
    super.didUpdateWidget(oldWidget);
  }

  @override
  void didChangeDependencies() {
    // TODO: implement didChangeDependencies
    super.didChangeDependencies();
  }
}

typedef ClickDefineBtn = Function();

class FCSkinManagerView extends StatefulWidget {
  final List<FUBeautyParam> skinList;
  final ClickDefineBtn? clickDefineBtn;
  FCSkinManagerView(this.skinList, {this.clickDefineBtn});
  @override
  FCSkinManagerViewState createState() => FCSkinManagerViewState();
}

class FCSkinManagerViewState extends State<FCSkinManagerView> {
  double _value = 0;
  int selectInt = 0;
  FUBeautyParam? currentParam;
  @override
  Widget build(BuildContext context) {
    if (currentParam == null) {
      currentParam = widget.skinList.first;
      _value = (currentParam!.mValue ?? 0) * 100;
    }
    return Container(
      color: Color(0xFFEEEEEE),
      // color: Colorshite,
      // color: Colors.transparent,
      padding: EdgeInsets.only(top: 10),
      child: Column(
        children: [
          Container(
            height: 80,
            child: Row(
              children: [
                Container(
                    padding: EdgeInsets.only(left: 20),
                    width: 100,
                    child: Text(
                      widget.skinList[selectInt].mTitle ?? '',
                      style: TextStyle(color: Color(0xFFFB5C9A), fontSize: 14),
                    )),
                Expanded(
                    child: SliderTheme(
                  data: SliderTheme.of(context).copyWith(
                    valueIndicatorColor: Color(0xFFFB5C9A),
                  ),
                  child: Slider(
                      min: 0,
                      divisions: 100,
                      max: 100,
                      activeColor: Color(0xFFFB5C9A),
                      inactiveColor: Colors.white,
                      label: '$_value',
                      onChangeEnd: (value) {
                        print(value);
                        widget.skinList[selectInt].mValue = value * 0.01;
                        FUManager.changeBeautyParam(widget.skinList[selectInt]);
                      },
                      value: _value,
                      onChanged: (newValue) {
                        setState(() {
                          _value = double.tryParse(newValue.toStringAsFixed(1)) ?? 0.0;
                        //  _value =( NumUtil.getNumByValueDouble(newValue, 1) ?? 0).toDouble();
                        //  _value = NumUtil.getNumByValueDouble(newValue  , 1)?.toDouble() ?? 0.0;
                        });
                      }),
                )),
                SizedBox(
                  width: 30,
                ),
              ],
            ),
          ),
          Expanded(
              child: Container(
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                GestureDetector(
                  onTap: () {
                    currentParam = null;
                    selectInt = 0;
                    // FUConfig.getSkinDefine();
                    FUConfig.getSkinDefine().forEach((element) {
                      FUManager.setSkinBeautyParam(element);
                    });

                    if (widget.clickDefineBtn != null) {
                      widget.clickDefineBtn!();
                    }
                  },
                  child: Container(
                    width: 80,
                    height: 80,
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        Image.asset(
                          'lib/assets/faceunity_default.png',
                          package: 'agora_rtc',
                        ),
                        Container(
                          padding: EdgeInsets.only(top: 10),
                          child: Text(
                            // S.of(context).Live_Default,
                            'Default',
                            style: TextStyle(color: Color(0xff333333), fontSize: 12),
                          ),
                        )
                      ],
                    ),
                  ),
                ),
                Container(
                  margin: EdgeInsets.only(top: 27),
                  color: Colors.black,
                  width: 2,
                  height: 25,
                ),
                Expanded(
                    child: Container(
                  height: 80,
                  margin: EdgeInsets.symmetric(horizontal: 20),
                  child: ListView.builder(
                    //  padding: EdgeInsets.all(value$),
                    itemCount: widget.skinList.length,
                    scrollDirection: Axis.horizontal,
                    itemBuilder: (BuildContext context, int index) {
                      return InkWell(
                        child: Container(
                          width: 60,
                          child: Image.asset(
                            'lib/assets/' +
                                (widget.skinList[index].image ?? '') +
                                (index == selectInt ? '_on' : '_off') +
                                '.png',
                            package: 'agora_rtc',
                          ),
                        ),
                        onTap: () {
                          if (mounted) {
                            setState(() {
                              selectInt = index;
                              _value =
                                  widget.skinList[selectInt].mValue ?? 0 * 100;
                            });
                          }
                        },
                      );
                    },
                  ),
                ))
              ],
            ),
          ))
        ],
      ),
    );
  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
  }

  @override
  void dispose() {
    // TODO: implement dispose
    super.dispose();
  }

  @override
  void didUpdateWidget(FCSkinManagerView oldWidget) {
    // TODO: implement didUpdateWidget
    super.didUpdateWidget(oldWidget);
  }

  @override
  void didChangeDependencies() {
    // TODO: implement didChangeDependencies
    super.didChangeDependencies();
  }
}




class FCFilterManagerView extends StatefulWidget {
  final List<FUBeautyParam> filterList;
  FCFilterManagerView(this.filterList);
  @override
  FCFilterManagerViewState createState() => FCFilterManagerViewState();
}

class FCFilterManagerViewState extends State<FCFilterManagerView> {
  // int selectInt = 0;
  // List skins = [
  //   'Default',
  //   'Nature',
  //   'Simple sense',
  //   'White',
  //   'Pink',
  //   'Cool',
  // ];
  @override
  Widget build(BuildContext context) {
    return Container(
      color: Color(0xFFEEEEEE),
      // color: Colors.transparent,
      // padding: EdgeInsets.only(top: 10),
      height: 80,
      padding: EdgeInsets.symmetric(horizontal: 20, vertical: 25),
      child: ListView.builder(
        //  padding: EdgeInsets.all(value$),
        itemCount: widget.filterList.length,
        scrollDirection: Axis.horizontal,
        itemBuilder: (BuildContext context, int index) {
          return InkWell(
            child: Container(
              margin: EdgeInsets.symmetric(horizontal: 10),
              width: 60,
              child: Column(
                children: [
                  Container(
                    width: 53,
                    height: 63,
                    margin: EdgeInsets.only(bottom: 10),
                    decoration: BoxDecoration(
                        // color: Color(0xFF17002D),
                        border: Border.all(
                            width: widget.filterList[index].isSeleted ?? false
                                ? 1.5
                                : 1.5,
                            color: widget.filterList[index].isSeleted ?? false
                                ? Color(0xFFFB5C9A)
                                : Colors.transparent),
                        borderRadius: BorderRadius.circular(5)),
                    child: index == 0
                        ? Center(
                            child: Image.asset(
                              'lib/assets/faceunity_default.png',
                              package: 'agora_rtc',
                            ),
                          )
                        : Container(
                            child: ClipRRect(
                            borderRadius: BorderRadius.circular(5),
                            child: Image.asset(
                              'lib/assets/' +
                                  (widget.filterList[index].image ?? '') +
                                  '.png',
                              fit: BoxFit.fill,
                              package: 'agora_rtc',
                            ),
                          )),
                  ),
                  Text(
                    widget.filterList[index].mTitle ?? '',
                    style: TextStyle(
                        color: Color(0xFF000000),
                            
                        fontSize: 12),
                  )
                ],
              ),
            ),
            onTap: () {
              widget.filterList.forEach((element) {
                element.isSeleted = false;
              });
              widget.filterList[index].isSeleted = true;
              FUManager.changeBeautyParam(widget.filterList[index]);
              if (mounted) {
                setState(() {
                  // selectInt = index;
                });
              }
            },
          );
        },
      ),
    );
  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
  }

  @override
  void dispose() {
    // TODO: implement dispose
    super.dispose();
  }

  @override
  void didUpdateWidget(FCFilterManagerView oldWidget) {
    // TODO: implement didUpdateWidget
    super.didUpdateWidget(oldWidget);
  }

  @override
  void didChangeDependencies() {
    // TODO: implement didChangeDependencies
    super.didChangeDependencies();
  }
}


